#!/usr/bin/env python
"""
Provide the class Garbage which is used to pad files and strings with
text advertising itself as "meaningless garbage".  Use to pad messages
prior to encryption.  If this method of padding is used, compression
should be turned off.

Every exception raised in the module is of class GarbageError, derived
from Exception class.

"""

__version__ = "$Revision: 1.6 $"
__author__ = "Peter Hendrickson <pdh@wiredyne.com>"

# License:  This code is in the public domain.  Any of this code may be
#           freely reproduced or adapted, even without mentioning the
#           source.

# Warranty: IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR
#           DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
#           DAMAGES ARISING OUT OF THE USE OF THIS CODE, EVEN IF THE
#           AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#           THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
#           INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#           MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE
#           CODE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THERE
#           IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
#           SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

import random
import types
import StringIO

class GarbageError(Exception):
    pass

class Garbage:
    """Pad files and strings with meaningless garbage text.

    State is set with __init__ method.  No variables should be
    tweaked directly.

    max_payload(): largest text that can be padded
    padfile(filename): pad filename to target size
    padstring(string): return padded version of string

    """
    def __init__(self, padded_size=None, line_len=None, begin_tag=None,
                 end_tag=None, eol=None, charpool=None):
        """Initialize garbage object with certain values.

        Keyword arguments:
        padded_size: Final size of padded object
        line_len: Length of line not counting end of line chars
        begin_tag: Text of starting tag (without end of line chars)
        end_tag: Text of ending tag (without end of line chars)
        eol: String of chars to be used for terminating lines
        charpool: String representing set of characters to use for padding

        Reasonable defaults are assigned to all unspecified values.
        
        """
        if padded_size is None:
            self.padded_size = 22 * 1024
        else:
            self.padded_size = padded_size

        if (type(self.padded_size) is not types.IntType) and \
           (type(self.padded_size) is not types.LongType):
            raise GarbageError("self.padded_size must be some kind of " + \
                                   "integer.")

        if line_len is None:
            self.line_len = 72
        else:
            self.line_len = line_len

        if (type(self.line_len) is not types.IntType) and \
           (type(self.line_len) is not types.LongType):
            raise GarbageError("self.line_len must be some kind of " + \
                                   "integer.")

        if begin_tag is None:
            self.begin_tag = "-----BEGIN MEANINGLESS GARBAGE-----"
        else:
            self.begin_tag = begin_tag

        if type(self.begin_tag) is not types.StringType:
            raise GarbageError("self.begin_tag must be a string.")

        if end_tag is None:
            self.end_tag = "-----END MEANINGLESS GARBAGE-----"
        else:
            self.end_tag = end_tag

        if type(self.end_tag) is not types.StringType:
            raise GarbageError("self.end_tag must be a string.")

        if eol is None:
            self.eol = "\n"
        else:
            self.eol = eol

        if type(self.eol) is not types.StringType:
            raise GarbageError("self.eol must be a string.")

        if charpool is None:
            self.charpool = "ABCDEFGHIJKLMNOPQRSTUVWXYZ" + \
                            "abcdefghijklmnopqrstuvwxyz" + \
                            "0123456789+/"
        else:
            self.charpool = charpool

        if type(self.eol) is not types.StringType:
            raise GarbageError("self.eol must be a string.")

        if len(self.eol) > self.line_len:
            raise GarbageError("The end of line sequence is longer " + \
                                   "than the number of chars in the line " + \
                                   "itself.")

        if self.max_payload() < 0:
            raise GarbageError("self.padded_size is too small to be " + \
                                   "padded as directed by the other settings.")

    def wrapper_size(self):
        """Return size of tags and some necessary line endings."""
        return len(self.eol + self.begin_tag + self.eol +
                   self.end_tag + self.eol)

    def min_padding(self):
        """Return smallest possible amount of padding text."""
        # An eol sequence is required because it has to be
        # complete or not present.  Fractions of the eol
        # sequence can't be used.
        return self.wrapper_size() + len(self.eol)

    def max_payload(self):
        """Return largest possible payload with given settings."""
        return self.padded_size - self.min_padding()

    def garbage_line(self, length):
        """Return a line of garbage characters terminated by self.eol.

        Random characters from self.charpool are returned, plus a
        self.eol sequence.  Total length is length characters long.
        If length is smaller than the length of self.eol,
        GarbageError is returned.
        
        """
        if length < len(self.eol):
            raise GarbageError("Length of %d characters is " % (length) + \
                                   "too small to be added")

        garbage = ''

        for i in xrange(length - len(self.eol)):
            garbage = garbage + random.choice(self.charpool)
        garbage = garbage + self.eol

        return garbage

    def write_garbage_guts(self, fileobj, length):
        """Write length bytes of garbage chars to fileobj.

        Each line is terminated with self.eol.  The self.eol
        characters count towards the total number of characters
        appended.
        
        """
        if length < len(self.eol):
            raise GarbageError("Additional padding too small to be added.")

        # Relationships:
        # length = (fully_padded_line * full_lines) + partial_line

        fully_padded_line = self.line_len + len(self.eol)
        full_lines = length // fully_padded_line
        partial_line = length % fully_padded_line

        if partial_line >= len(self.eol):
            for i in xrange(full_lines):
                fileobj.write(self.garbage_line(fully_padded_line))
            fileobj.write(self.garbage_line(partial_line))
        elif partial_line == 0:
            for i in xrange(full_lines):
                fileobj.write(self.garbage_line(fully_padded_line))
        else:
            # The tricky case: eol is more than one character so we
            # can't just add it to increment one character.  Thus,
            # we need to drop some chars from the previous line to
            # make the count come out right.
            true_full_lines = full_lines - 1
            for i in xrange(true_full_lines):
                fileobj.write(self.garbage_line(fully_padded_line))
            truncated_line = fully_padded_line - (len(self.eol) - partial_line)
            fileobj.write(self.garbage_line(truncated_line))
            fileobj.write(self.eol)

    def write_garbage(self, fileobj):
        """Pad fileobj to self.padded_size."""
        fileobj.seek(0, 2) # Seek to end
        content_length = fileobj.tell()

        if content_length > self.max_payload():
            raise GarbageError("File is too big to be padded to " + \
                               "desired size")

        padding_needed = self.padded_size - content_length

        fileobj.write(self.eol + self.begin_tag + self.eol)

        guts_size = padding_needed - self.wrapper_size()
        self.write_garbage_guts(fileobj, guts_size)

        fileobj.write(self.end_tag + self.eol)

    def padfile(self, filename):
        """Pad filename."""
        # Some systems might change line endings.  We care about every
        # byte, however, so binary mode is required.
        fileobj = open(filename, "ab")
        self.write_garbage(fileobj)
        fileobj.close()

    def padstring(self, string):
        """Return padded string."""
        fakefile = StringIO.StringIO(string)
        self.write_garbage(fakefile)
        padded = fakefile.getvalue()
        fakefile.close()

        return padded
