#!/usr/bin/env python
"""
usage: pad.py filename [padded size]

Pad out a message file to padded size with nice markers so recipients
will understand they can ignore the padding characters.

The primary use is for files containing messages which will be
encrypted.  This makes it easy to send out fixed sized messages which
deny an eavesdropper some useful clues about the contents.

It is a bad idea to compress a message before encrypting if it is
padded in this way.  The "-z 0" argument to gpg will turn off
compression.

"""

__version__ = "$Revision: 1.9 $"
__author__ = "Peter Hendrickson <pdh@wiredyne.com>"

# License:  This code is in the public domain.  Any of this code may be
#           freely reproduced or adapted, even without mentioning the
#           source.

# Warranty: IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR
#           DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
#           DAMAGES ARISING OUT OF THE USE OF THIS CODE, EVEN IF THE
#           AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#           THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES,
#           INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#           MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE
#           CODE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THERE
#           IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
#           SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

import sys
import os
import stat
import Garbage

def main():
    filename, padded_size = process_args()

    if padded_size is None:
            gbg = Garbage.Garbage()
    else:
            gbg = Garbage.Garbage(padded_size=padded_size)
            
    filelength = os.path.getsize(filename)
    if filelength > gbg.max_payload():
        sys.stderr.write("%s is too big to be " % (filename) + \
			 "padded to the desired size.\n")
        sys.exit(1)

    try:
        gbg.padfile(filename)
    except:
        sys.stderr.write("Unexpected Error: %s (%s)\n" % sys.exc_info()[:2])
        sys.exit()

def process_args():
    if (len(sys.argv) > 3) or len(sys.argv) == 1:
        sys.stderr.write("usage: %s filename [desired size]\n" % \
                         (sys.argv[0]))
        sys.exit(1)

    file = sys.argv[1]

    if not os.access(file, os.F_OK):
        sys.stderr.write("%s does not exist.\n" % (file))
        sys.exit(1)

    status = os.stat(file)
    if not stat.S_ISREG(status[0]):
        sys.stderr.write("%s is not a regular file.\n" % (file))
        sys.exit(1)

    if not os.access(file, os.W_OK):
        sys.stderr.write("Cannot append to %s\n" % (file))
        sys.exit(1)

    padded_size = None
    if len(sys.argv) == 3:
        try:
            padded_size = int(sys.argv[2])
        except:
            sys.stderr.write("%s is not an integer.\n" % \
                             (repr(sys.argv[2])))
            sys.exit(1)

    return file, padded_size

main()
